// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Filename.h: --- a class representing a file name
//
//	DESCRIPTION:
//		Instances of class Filename represent a valid file name
//
//	CHANGE HISTORY:
//		UL, 1998-03-25: 1.0.0: - first version
//		UL, 1998-10-07: 1.1.0: - added explicit operator<<
//
//	AUTHOR:
//		U. Lammers (UL), Wed Mar 25 16:52:23 MET 1998
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/src/utils/Filename.h,v 1.3 2001/09/26 13:46:39 ulammers Exp $
//
#ifndef FILENAME_H
#define FILENAME_H

#include <string>
#include <iosfwd>

class FileName : public std::string {
public:
	FileName() : std::string() {}
	FileName(const char *name) : std::string(name) {}
	FileName(const std::string &name) : std::string(name) {}

	int blockNr() const;
		//
		//	for block specification appended to the file name in either
		//	of the FTOOLS forms:
		//		name+x	or  name[x]  or  name
		//	the function returns the numeric value of 'x' - 1 is returned
		//	if the last form is used - if 'x' does not represent a
		//	numeric value, -1 is returned

	std::string blockName() const;
		//
		//	for a block specification appended to 

	std::string chopBlockSpec() const;
		//
		//  chop-off block specification from a file name
		//

	std::string basename() const;
		//
		//	give the basename component of a filename
		//

	~FileName() {}
}; // FileName

std::ostream &operator<<(std::ostream &os, const FileName &fn);

#endif

