! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!   filtertestf90.f90: test the F90 wrapper for filter class
!
!   UL, Wed Mar 25 13:50:58 MET 1998
!
!   $Header: /sasbuild/repositories/sasdev/sas/packages/selectlib/test/filtertestf90.f90,v 1.8 2002/07/31 12:27:36 ulammers Exp $
!
program filtertestf90
    use dal
    use datafilter
    use exprcolgen
    use errorhandling

    type(DataSetT) :: set
    type(TableT)   :: tab
    type(ColumnT)  :: col
    integer(int8), dimension(:), pointer  :: flagData
    integer(int32), dimension(:), pointer :: fromRow, toRow
    integer        :: n

    !
    !   read file `filtertest.ds' created by filtertestcxx
    !
    set = dataSet('filtertest.ds', Modify)
    tab = table(set, 'SPATIAL')

    !
    !   filter the table with a box region - selects everything
    !
    call initFilter('box(128.5,128.5,128,128,0,RAWX,RAWY)')
    if (doSelect(tab, 'FLAG') .ne. numberOfRows(tab)) &
        call fatal('filtertestf90','filter1: wrong number of selected photons')

    col = column(tab, 'FLAG', Read)
    flagData => int8Data(col)
    do n=0, numberOfRows(tab)-1
        if (flagData(n) .eq. 0) &
            call fatal('filtertestf90', 'filter2: wrong selection status')
    enddo

    !
    !   filter the table with a box region - selects nothing
    !
    call initFilter('box(128.5,128.5,0,0,0,RAWX,RAWY)')
    if (doSelect(tab, 'FLAG') .ne. 0) &
        call fatal('filtertestf90','filter3: wrong number of selected photons')

    col = column(tab, 'FLAG', Read)
    flagData => int8Data(col)
    do n=0, numberOfRows(tab)-1
        if (flagData(n) .ne. 0) &
            call fatal('filtertestf90', 'filter4: wrong selection status')
    enddo

    !
    !   filter the table with another region filter - more complex
    !
    call initFilter('((100*sin(RAWX/256*360*#RAD) > RAWY-200) &&     &
                    &(100*sin(RAWX/256*360*#RAD) < RAWY-150)) ||     &
                    &((100*sin(RAWY/256*360*#RAD) > RAWX-200) &&     &
                    &(100*sin(RAWY/256*360*#RAD) < RAWX-150))')

    if (doSelect(tab, fromRow, toRow) .ne. 19361) &
        call fatal('filtertestf90','filter5: wrong number of selected photons')

    if (size(fromRow) .ne. 501 .or. size(toRow) .ne. 501) &
        call fatal('filtertestf90', &
                   'filter6: we should have 501 blocks of selected rows')

    if (fromRow(0) .ne. 152 .or. toRow(0) .ne. 201 &
        .or. &
        fromRow(39) .ne. 10217 .or. toRow(39) .ne. 10239 &
        .or. &
        fromRow(453) .ne. 60687 .or. toRow(453) .ne. 60714 &
        .or. &
        fromRow(500) .ne. 65429 .or. toRow(500) .ne. 65478) &
        call fatal('filtertestf90', 'filter7: row ranges are wrong')

    call selectReleaseMemory(fromRow, toRow)

    !
    !   check column generation
    !
    call initExprColGen('sqrt((RAWX-1)**2 + (RAWY-1)**2) - DIST', 'DELTA_D',&
                        Real32)
    call doExprColGen(tab, tab)

    call initFilter('DELTA_D>1.e-8')  ! should be false for all rows
    if (doFilter(tab) /= 0) &
        call fatal('filtertestf90', 'filter8: non-zero number of rows')    

    call release(set)
                        
    call message(AppMsg, Verbose, 'all tests successfully passed')
end program

